#include <stddef.h>
#include <stdint.h>

#include "serial.h"
#include "printk.h"
#include "device_tree.h"
#include "led.h"
#include "delay.h"
#include "timer.h"
#include "interrupts.h"
#include "bcm2835_periph.h"
#include "mmio.h"
#include "hardware_detect.h"
#include "gic-400.h"
#include "shell.h"

/* Homework 5 */
#define VERSION 5

uint32_t hardware_revision=0;

/* set in hardware_detect.c */
uint32_t io_base;

void kernel_main(uint32_t r0, uint32_t r1, uint32_t r2) {

	(void) r0;	/* Ignore boot method */

	/* Detect Hardware */
	hardware_detect((void *)r2);

	/* Setup Model-specific Variables based on harware revision */
	hardware_setup_vars();

	/* Initialize Hardware */
	uart_init();
	led_init();
	timer_init();

	/* Enable Pi4 gic400 Interrupt Controller */

	if (hardware_type==RPI_MODEL_4B) {
		printk("Enabling GIC-400 on Pi4B\n");
		gic400_init((void *)0xFF840000UL);
	}

	/* enable interrupts on CPU */
	enable_interrupts();

	/* Delay to allow time for serial port to settle */
	/* So we can actually see the output on the terminal */

	printk("\nWaiting for serial port to be ready (press any key)\n");
	uart_getc();

	/* Print boot messages */


	/* Print hardware version */
	hardware_print_model(r1);

	/* switch to userspace and enter our shell */

	printk("Entering userspace\n");

	/* Setup user stack */
#if 0
	asm volatile(
		"msr CPSR_c, #0xDF\n" /* System mode, like user but privldg */
		"mov sp, #0x00002000\n"
		"msr CPSR_c, #0xD3\n" /* Back to Supervisor mode */
		: /* output */
		: /* input */
		: "memory");      /* clobbers */


	/* Switch to userspace */

	asm volatile(
		"mov r0, #0x10\n"
		"msr SPSR, r0\n"
		"mov lr, %[shell]\n"
		"movs pc,lr\n"
		: /* output */
		: [shell] "r"(shell) /* input */
		: "r0", "lr", "memory");        /* clobbers */

#endif

	shell();


}
